@@ -1,17 +1,14 @@
 use anyhow::Result;
 use std::convert::TryFrom;
 use std::net::SocketAddr;
-use std::path::PathBuf;
 use std::sync::Arc;
-use thiserror::Error;
 use tokio::net::TcpStream;
-use tokio_rustls::client::TlsStream;
 use tokio_rustls::rustls::{self, ClientConfig, RootCertStore};
 use tokio_rustls::TlsConnector;
 
-use crate::certstore;
-use crate::client;
+use crate::{client, TlsStream};
 use crate::resolve_address;
+use crate::{certificates, EasyTlsError};
 
 fn get_client_config(root_store: RootCertStore) -> ClientConfig {
     rustls::ClientConfig::builder()
@@ -20,32 +17,28 @@ fn get_client_config(root_store: RootCertStore) -> ClientConfig {
         .with_no_client_auth()
 }
 
-/// Represents errors that may be returned through [`TlsClient`] usage.
-#[derive(Error, Debug)]
-pub enum TlsClientError {
-    #[error("Address resolution error: {0}:{1}")]
-    ResolutionFailure(String, u16),
-}
-
 /// Primary class for creating and connecting clientside TLS sockets
 pub struct TlsClient {
     host: String,
     address: SocketAddr,
-    cafile: Option<PathBuf>,
+    cafile: Option<String>,
 }
 
 impl TlsClient {
-    /// Returns a new [`TlsClient`] struct, this currently using the blocking
-    /// address resolution method upon creation. In future changes I will make
-    /// this use asynchronous resolution, but resolution will still occur at
+    /// Returns a new [`TlsClient`] struct, address resolution will occur at
     /// creation. Connection attempts are performed later with calls to
     /// [`TlsClient::connect`]
-    pub async fn new<T>(host: T, port: u16) -> Result<Self>
+    pub async fn new<T>(host: T) -> Result<Self>
     where
         T: ToString,
     {
         let host = host.to_string();
-        let address = resolve_address(host.as_str(), port)?;
+        let address = resolve_address(host.as_str()).await?;
+
+        let host = match host.to_string().split_once(":") {
+            Some((hostname, _)) => hostname.to_string(),
+            None => return Err(EasyTlsError::ResolutionFailure(host).into()),
+        };
 
         Ok(TlsClient {
             host,
@@ -56,7 +49,7 @@ impl TlsClient {
 
     /// This function can be called if you need to set or use a certificate
     /// authority file instead of the defaults
-    pub async fn set_cafile(&mut self, cafile: PathBuf) {
+    pub async fn set_cafile(&mut self, cafile: String) {
         self.cafile = Some(cafile)
     }
 
@@ -64,11 +57,12 @@ impl TlsClient {
     /// A [`TlsStream<TcpStream>`] handle will be returned on success
     pub async fn connect(&self) -> Result<TlsStream<TcpStream>> {
         let cafile = &self.cafile;
-        let root_store = certstore::get_root_store(cafile)?;
+        let root_store = certificates::get_root_store(cafile.clone()).await?;
         let config = client::get_client_config(root_store);
         let connector = TlsConnector::from(Arc::new(config));
         let tcp = TcpStream::connect(self.address).await?;
         let domain = rustls::ServerName::try_from(self.host.as_str())?;
-        Ok(connector.connect(domain, tcp).await?)
+        let stream = connector.connect(domain, tcp).await?;
+        Ok(stream.into())
     }
 }
